@@ -359,6 +359,18 @@ Q_SIGNALS:
      */
     void changed(bool state);
 
+    /**
+     * Indicate that the state of the modules contents matches the default
+     * settings.
+     *
+     * This signal is emitted whenever the state of the configuration
+     * shown in the module changes. It allows the module container to
+     * keep track of defaults.
+     *
+     * @since 5.65
+     */
+    void defaulted(bool state);
+
     /**
      * Indicate that the module's quickhelp has changed.
      *
@@ -420,7 +432,7 @@ protected:
      * Default: shows a "Use Defaults" button.
      *
      * Apply: in kcontrol this will show an "Apply" and "Reset" button,
-     *        in kcmshell this will show an "Ok", "Apply" and "Cancel" button.
+     *        in kcmshell this will show an "Ok", "Apply", "Reset" and "Cancel" button.
      *
      * If Apply is not specified, kcmshell will show a "Close" button.
      *
@@ -453,12 +465,27 @@ protected:
      */
     bool managedWidgetChangeState() const;
 
+    /**
+     * Returns the defaulted state of automatically managed widgets in this dialog
+     *
+     * @since 5.65
+     */
+    bool managedWidgetDefaultState() const;
+
     /**
      * Call this method when your manually managed widgets change state between
      * changed and not changed
      */
     void unmanagedWidgetChangeState(bool);
 
+    /**
+     * Call this method when your manually managed widgets change state between
+     * defaulted and not defaulted
+     *
+     * @since 5.65
+     */
+    void unmanagedWidgetDefaultState(bool);
+
 private:
     KCModulePrivate *const d;
 };
