@@ -28,11 +28,11 @@
 #include "krecentfilesaction.h"
 #include "krecentfilesaction_p.h"
 
-#include <QtCore/QFile>
-#include <QDesktopWidget>
+#include <QFile>
 #include <QDir>
+#include <QGuiApplication>
 #include <QMenu>
-#include <QComboBox>
+#include <QScreen>
 
 #include <kconfig.h>
 #include <kconfiggroup.h>
@@ -81,7 +81,7 @@ void KRecentFilesActionPrivate::init()
     clearSeparator = q->menu()->addSeparator();
     clearSeparator->setVisible(false);
     clearSeparator->setObjectName(QStringLiteral("separator"));
-    clearAction = q->menu()->addAction(i18n("Clear List"), q, SLOT(clear()));
+    clearAction = q->menu()->addAction(i18n("Clear List"), q, &KRecentFilesAction::clear);
     clearAction->setObjectName(QStringLiteral("clear_action"));
     clearAction->setVisible(false);
     q->setEnabled(false);
@@ -123,19 +123,19 @@ static QString titleWithSensibleWidth(const QString &nameValue, const QString &v
     // action titles to be bigger than that
     // Since we do not know in which screen we are going to show
     // we choose the min of all the screens
-    const QDesktopWidget desktopWidget;
     int maxWidthForTitles = INT_MAX;
-    for (int i = 0; i < desktopWidget.screenCount(); ++i) {
-        maxWidthForTitles = qMin(maxWidthForTitles, desktopWidget.availableGeometry(i).width() * 3 / 4);
+    const auto screens = QGuiApplication::screens();
+    for (QScreen *screen : screens) {
+        maxWidthForTitles = qMin(maxWidthForTitles, screen->availableGeometry().width() * 3 / 4);
     }
     const QFontMetrics fontMetrics = QFontMetrics(QFont());
 
-    QString title = nameValue + QStringLiteral(" [") + value + QLatin1Char(']');
-    if (fontMetrics.width(title) > maxWidthForTitles) {
+    QString title = nameValue + QLatin1String(" [") + value + QLatin1Char(']');
+    const int nameWidth = fontMetrics.boundingRect(title).width();
+    if (nameWidth > maxWidthForTitles) {
         // If it does not fit, try to cut only the whole path, though if the
         // name is too long (more than 3/4 of the whole text) we cut it a bit too
         const int nameValueMaxWidth = maxWidthForTitles * 3 / 4;
-        const int nameWidth = fontMetrics.width(nameValue);
         QString cutNameValue, cutValue;
         if (nameWidth > nameValueMaxWidth) {
             cutNameValue = fontMetrics.elidedText(nameValue, Qt::ElideMiddle, nameValueMaxWidth);
@@ -144,7 +144,7 @@ static QString titleWithSensibleWidth(const QString &nameValue, const QString &v
             cutNameValue = nameValue;
             cutValue = fontMetrics.elidedText(value, Qt::ElideMiddle, maxWidthForTitles - nameWidth);
         }
-        title = cutNameValue + QStringLiteral(" [") + cutValue + QLatin1Char(']');
+        title = cutNameValue + QLatin1String(" [") + cutValue + QLatin1Char(']');
     }
     return title;
 }
@@ -172,7 +172,8 @@ void KRecentFilesAction::addUrl(const QUrl &_url, const QString &name)
 #endif
 
     // remove file if already in list
-    foreach (QAction *action, selectableActionGroup()->actions()) {
+    const auto lstActions = selectableActionGroup()->actions();
+    for (QAction *action : lstActions) {
         const QString urlStr = d->m_urls[action].toDisplayString(QUrl::PreferLocalFile);
 #ifdef Q_OS_WIN
         const QString tmpFileName = url.isLocalFile() ? QDir::toNativeSeparators(urlStr) : urlStr;
