@@ -32,10 +32,13 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <QCheckBox>
 #include <QKeyEvent>
 #include <QLabel>
-#include <QLayout>
 #include <QDesktopWidget>
 #include <QPushButton>
+#include <QScreen>
 #include <QTextBrowser>
+#include <QHBoxLayout>
+#include <QRegularExpression>
+#include <QVBoxLayout>
 
 #include <kconfig.h>
 #include <kconfiggroup.h>
@@ -83,16 +86,16 @@ void KTipDatabase::Private::addTips(const QString &tipFile)
 
     QByteArray data = file.readAll();
     QString content = QString::fromUtf8(data.constData(), data.size());
-    const QRegExp rx(QStringLiteral("\\n+"));
+    const QRegularExpression rx(QStringLiteral("\\n{2,}"));
 
     int pos = -1;
-    while ((pos = content.indexOf(QStringLiteral("<html>"), pos + 1, Qt::CaseInsensitive)) != -1) {
+    while ((pos = content.indexOf(QLatin1String("<html>"), pos + 1, Qt::CaseInsensitive)) != -1) {
         /**
          * To make translations work, tip extraction here must exactly
          * match what is done by the preparetips5 script.
          */
         QString tip = content
-                      .mid(pos + 6, content.indexOf(QStringLiteral("</html>"), pos, Qt::CaseInsensitive) - pos - 6)
+                      .mid(pos + 6, content.indexOf(QLatin1String("</html>"), pos, Qt::CaseInsensitive) - pos - 6)
                       .replace(rx, QStringLiteral("\n"));
 
         if (!tip.endsWith('\n')) {
@@ -100,7 +103,7 @@ void KTipDatabase::Private::addTips(const QString &tipFile)
         }
 
         if (tip.startsWith('\n')) {
-            tip = tip.mid(1);
+            tip.remove(0, 1);
         }
 
         if (tip.isEmpty()) {
@@ -159,7 +162,7 @@ void KTipDatabase::nextTip()
 
     d->currentTip += 1;
 
-    if (d->currentTip >= (int) d->tips.count()) {
+    if (d->currentTip >= d->tips.count()) {
         d->currentTip = 0;
     }
 }
@@ -290,10 +293,12 @@ KTipDialog::KTipDialog(KTipDatabase *database, QWidget *parent)
         resize(520, 280);
         QSize sh = size();
 
-        QRect rect = QApplication::desktop()->screenGeometry(QCursor::pos());
-
-        move(rect.x() + (rect.width() - sh.width()) / 2,
-             rect.y() + (rect.height() - sh.height()) / 2);
+        QScreen *screen = QGuiApplication::screenAt(QCursor::pos());
+        if (screen) {
+            const QRect rect = screen->geometry();
+            move(rect.x() + (rect.width() - sh.width()) / 2,
+                    rect.y() + (rect.height() - sh.height()) / 2);
+        }
     }
 
     KSeparator *sep = new KSeparator(Qt::Horizontal);
